unit IIRO8U;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Registry, ExtCtrls ;

type
  TMainFrm = class(TForm)
    HeaderLabel: TLabel;
    GroupBox1: TGroupBox;
    ExitButton: TBitBtn;
    Memo1: TMemo;
    CardNameLabel: TLabel;
    BeginSample: TButton;
    RunTimer: TTimer;
    RelayOut: TPanel;
    RelayRead: TPanel;
    OptoIn: TPanel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    AddressLabel: TLabel;
    procedure ExitButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RunTimerTimer(Sender: TObject);
    procedure BeginSampleClick(Sender: TObject);

  private
  { Private declarations }
  public
  { Public declarations }
  end;


var
  MainFrm: TMainFrm;

implementation

{$R *.DFM}

uses
  AIOWDM;

function IntToBin(I: Integer; Bits: Integer): AnsiString; forward;

//global variables for internal use
var
  Siz: Integer = 8;
  NumCards, CardNum: Integer;
  RunFlag:Boolean;
  TestIndex: Integer;

procedure TMainFrm.FormCreate(Sender: TObject);
var
    found: Boolean;

    DeviceID, Base: LongWord;
    iCard: Integer;

begin
    found := False;
    RunFlag := True;

    //Use WDM driver num cards.
    NumCards := GetNumCards();
    if NumCards = 0 then begin
        Memo1.Text := 'No cards found. The cards may not be installed, or they may be installed using a driver other than AIOWDM.sys.';
    end;

    CardNum := -1; //No card found yet.
    for iCard := 0 to NumCards-1 do begin
      if QueryCardInfo(iCard, @DeviceID, @Base, nil, nil) = 0 then
        Continue //If the query didn't work, just skip that card.
      ;

      case DeviceID of
        $0F00: begin
          CardNameLabel.Caption := 'PCI-IIRO-8 Digital Input/Relay Output Card';
          HeaderLabel.Caption := 'Windows IIRO-8/16 Sample';
          Siz := 8;
        end;
        $0F08: begin
          CardNameLabel.Caption := 'PCI-IIRO-16 Digital Input/Relay Output Card';
          HeaderLabel.Caption := 'Windows IIRO-8/16 Sample';
          Siz := 16;
        end;
        $0DC8: begin
          CardNameLabel.Caption := 'PCI-IDIO-16 Solid-State Input/Output Card';
          HeaderLabel.Caption := 'Windows IIRO-8/16 Sample';
          Siz := 16;
        end;
        $0F02: begin
          CardNameLabel.Caption := 'PCIe-IIRO-8 Digital Input/Relay Output Card';
          HeaderLabel.Caption := 'Windows IIRO-8/16 Sample';
          Siz := 8;
        end;
        $0F09: begin
          CardNameLabel.Caption := 'PCIe-IIRO-16 Digital Input/Relay Output Card';
          HeaderLabel.Caption := 'Windows IIRO-8/16 Sample';
          Siz := 16;
        end;
        $0102: begin
          CardNameLabel.Caption := 'mPCIe-IIRO-8 Digital Input/Relay Output Card';
          HeaderLabel.Caption := 'Windows IIRO-8/16 Sample';
          Siz := 8;
        end;
        else Continue; //We don't recognize the card, so skip it.
      end;{case}

      found := true;
      CardNum := iCard;

      Break;
    end;

    if found then begin
      RelayOut.Caption := StringOfChar('0', Siz);
      RelayRead.Caption := StringOfChar('0', Siz);
      OptoIn.Caption := StringOfChar('0', Siz);
    end
    else begin
      HeaderLabel.Caption := 'Windows IIRO-8/16 Sample';
      CardNameLabel.Caption := 'No Card Found In System';
      Memo1.Lines.Clear;
      Memo1.Lines.Append('No Card Found.');
      Memo1.Lines.Append('This may mean no card is installed, or that NTioPCI.SYS is not installed.');
      //Memo1.Lines.Append('If you have a PCI card installed, please make sure you have run PCIFind.EXE.');
      RunFlag:=False;
      BeginSample.Enabled := False;
    end;
end;

procedure TMainFrm.ExitButtonClick(Sender: TObject);
begin
    Close;
end;

procedure TMainFrm.RunTimerTimer(Sender: TObject);
var
   value: array[0..2] of LongWord;
   bWrite: Boolean;

begin
    //We will be using the Relative Port functions, although absolute is also available.

    bWrite := (TestIndex mod 10) = 0; //We read on every timer tick, but only write about once per second.

    value[0] := 1 shl (TestIndex div 10);
    if bWrite then RelOutPortB(CardNum, 0, value[0]);
    value[1] := RelInPortB(CardNum, 0);
    value[2] := RelInPortB(CardNum, 1);
    if Siz > 8 then begin
      if bWrite then RelOutPortB(CardNum, 4, value[0] shr 8);
      value[1] := value[1] + InPortB(4) shl 8;
      value[2] := value[2] + InPortB(4 + 1) shl 8;
    end;
    RelayOut.Caption := IntToBin(Value[0], Siz);
    RelayRead.Caption := IntToBin(Value[1], Siz);
    OptoIn.Caption := IntToBin(Value[2], Siz);

    TestIndex := (TestIndex + 1) mod (Siz * 10);
end;

procedure TMainFrm.BeginSampleClick(Sender: TObject);
begin
    if RunTimer.Enabled=True then Begin
        RunTimer.Enabled:=False;
        BeginSample.Caption:='Perform I/O';
    end else
    begin
        RunTimer.Enabled:=True;
        BeginSample.Caption:='Stop I/O';
    end;
end;





function IntToBin(I: Integer; Bits: Integer): AnsiString;
var
  J: Integer;

begin
  Result := StringOfChar('0', Bits);
  for J := 0 to Bits-1 do
    Result[Bits - J] := Chr($30 + Ord((I and (1 shl J)) <> 0))
  ;
end;

end.
